/*	Filename:		BustaNetClient.h
 *	Description:	Busta' Sandwich client module.  Interfaces with game engine and server
 *
 */

#ifndef BUSTA_SANDWICH_CLIENT
#define BUSTA_SANDWICH_CLIENT

#include "SDL_net.h"
#include "BustaNetMessages.h"
#include "BustaNetInternal.h"

/**
 * @class BustaNetClient
 * @brief Client network interface to Busta'Sandwich
 *
 * Busta'Sandwich Client for networked multiplayer games.
 */
class BustaNetClient
{
	public:
		/**
		 * @brief Constructor
		 *
		 * Initializes variables
		 */
		BustaNetClient();

		/**
		 * @brief Destructor
		 *
		 * Close any open connections and shutdown SDLNet
		 */
		~BustaNetClient();

		/**
		 * @brief Initializes the client and prepares it for connection
		 *
		 * Initializes SDLNet.  Relays any error message back.
		 *
		 * @return Returns a cstring containing an error message if SDLNet cannot be initialized.
		 *	On no error, null is returned.
		 */
		char* init();

		/**
		 * @brief Connects to a server using a given port
		 *
		 * The client initializes a TCP connect with the server on the specified port.
		 *	On successful connection, an attempt to join the game is made.
		 *
		 * @param IP The IP address of the server to connect to.
		 * @param serverPort The port of the server to connect to.
		 *
		 * @return Returns a pointer to a connect_resp struct outlining the outcome of the connection
		 *	attempt.
		 */
		connect_resp* connect(char* IP, Uint16 serverPort);

		/**
		 * @brief Disconnects from the server
		 *
		 * The connection between the server and client is disconnected.  Subsequent connections may
		 *	be made using the client
		 */
		void disconnect();

		//MPGameState getMPGS()
		//getPriorityQ();
		//getSandwichQ();

		/**
		 * @brief Handles all incoming network data
		 *
		 * Processes all data currently queued on the TCP socket.  The data is partitioned out into
		 *	three message containers.  Each of these three message containers is accessible through
		 *	an accompanying method.
		 *
		 * @see getMPGS()
		 * @see getPriorityQ()
		 * @see getSandwichQ()
		 */
		void processNet();

		//void sendCharSelect(CHARACTER);
		//void sendCursor(Cursor cursor);

		/**
		 * @brief Notifies the server that the player has lost the game
		 *
		 * A message is sent to the server to notify it that the player has lost in the current
		 *	multiplayer game.
		 */
		void sendGameOver();

		/**
		 * @brief Tells the server which opponent the player wishes to view
		 *
		 * A message is sent to the server to tell it which opponent the player wishes to have a
		 *	detailed view of.  The detailed view contains information to display the entire
		 *	screen of the opponent rather than limited brief data.
		 *
		 * @param opponentID The identification of the opponent whom the player wishes to view
		 */
		void sendOpponentViewReq(int opponentID);

		//void sendSandwich(int opponentID, CompleteSandwich sandwich);
		//void sendState(GameState gs);

	private:
		IPaddress serverIP;			// IP address of the game server
		char* serverName;			// Name of the game server

		SDLNet_SocketSet socketset;	// Socketset containing server connection socket
		TCPsocket tcpsock;			// Socket connection to server

		bool debug;		// DEBUG NOTIFIER
};

#endif
