/*	Filename:		BustaNetMEssages.h
 *	Description:	Contains definitions of all net messages sent across the network.
 *
 */

// -----------------------------------------------------------------------------------------------
// ------------------------------- CLIENT MESSAGES -----------------------------------------------
// -----------------------------------------------------------------------------------------------

/**
 * @struct cs_connect_attempt BustaNetMessages.h
 *
 * @brief Client message to server attempting to join server game
 *
 * The message contains the client's preferred multiplayer game name.  The message acts as a notifier
 *	to the server that the client wishes to join the game.
 */
struct cs_connect_attempt
{
	char type;			/**< The message type */
	int length;			/**< The length of the entire message in bytes */
	char* clientName;	/**< The name of the client's player */
};

/**
 * @struct cs_character_select BustaNetMessages.h
 *
 * @brief Client sends a character selection
 *
 * This message is sent from a client to the server detailing the character selected by the player.
 *	Each player selects a character chef for a multiplayer game.  Once a selection is made, this
 *	message is sent to the server dictating which character was selected by the client.
 */
struct cs_character_select
{
	char type;			/**< The message type */
	int length;			/**< The length of the entire message in bytes */
	char character;		/**< The character the player has selected */
};

/**
 * @struct cs_cursor BustaNetMessages.h
 *
 * @brief Client moves the cursor during character selection
 *
 * The message contains the location of the cursor when the client is selecting a multiplayer
 *	character.  This notifies the server of the client's cursor position when selecting takes place.
 */
struct cs_cursor
{
	char type;			/**< The message type */
	int length;			/**< The length of the entire message in bytes */
	char cursorLoc;		/**< The location of the cursor */
};

/**
 * @struct cs_disconnect BustaNetMessages.h
 *
 * @brief Client disconnects from the server
 *
 * This message is a notification to the server that the client is disconnecting.
 */
struct cs_disconnect
{
	char type;			/**< The message type */
	int length;			/**< The length of the entire message in bytes */
};

/**
 * @struct cs_gamestate BustaNetMessages.h
 *
 * @brief Client sends the server its current in-game game state
 *
 * The client sends the player's game state to the server.  The game state includes all
 *	information about the player's actions and statistics.
 */
struct cs_gamestate
{
	char type;			/**< The message type */
	int length;			/**< The length of the entire message in bytes */
	// TBI
};

/**
 * @struct cs_gameover BustaNetMessages.h
 *
 * @brief Client sends a notification to the server that the player has lost
 *
 * This message is a notification to the server that the player has lost in the current multiplayer
 *	game.
 */
struct cs_gameover
{
	char type;			/**< The message type */
	int length;			/**< The length of the entire message in bytes */
};

/**
 * @struct cs_opponent_view_select BustaNetMessages.h
 *
 * @brief Client requests to be sent full game state information about a particular opponent
 *
 * The message contains the identification number of the opponent the client wishes to see full
 *	detail of.  A client who sends this message will get full game state updates for the specified
 *	opponent.
 */
struct cs_opponent_view_select
{
	char type;			/**< The message type */
	int length;			/**< The length of the entire message in bytes */
	int opponentID;		/**< The identification number of the specified opponent */
};

/**
 * @struct cs_ready BustaNetMessages.h
 *
 * @brief Client notifies the server that player is ready to transition out of the connect state
 *
 * A client sends this message when the player is ready to move to character selection.  Sending this
 *	message signifies that the client wishes to begin the game along with everyone else already
 *	connected to the server.
 */
struct cs_ready
{
	char type;			/**< The message type */
	int length;			/**< The length of the entire message in bytes */
};

/**
 * @struct cs_sandwich_send BustaNetMessages.h
 *
 * @brief Client sends a sandwich to an opponent player
 *
 * This message is sent to the server when a player sends a special sandwich to an opponent.
 */
struct cs_sandwich_send
{
	char type;			/**< The message type */
	int length;			/**< The length of the entire message in bytes */
	int targetPlayer;	/**< The identification number of the opponent client */
	// TBI - sandwich
};

// -----------------------------------------------------------------------------------------------
// ------------------------------- SERVER MESSAGES -----------------------------------------------
// -----------------------------------------------------------------------------------------------

/**
 * @struct sc_character_selected BustaNetMessages.h
 *
 * @brief Server notification to client of character selection
 *
 * The server sends this message to a client when another client has made a character selection.
 *	This allows the client to know when and what character was selected by an opponent.
 */
struct sc_character_selected
{
	char type;				/**< The message type */
	int length;				/**< The length of the entire message in bytes */
	int selectingPlayer;	/**< The identification number of the client who has selected */
	char character;			/**< The character selected by the player */
};

/**
 * @struct sc_character_select_mode BustaNetMessages.h
 *
 * @brief Server notification to client to begin selecting a character
 *
 * This message is sent from the server to a client to notify the client it is time to make
 *	a multiplayer character selection.
 */
struct sc_character_select_mode
{
	char type;				/**< The message type */
	int length;				/**< The length of the entire message in bytes */
};

/**
 * @struct sc_character_select_cursor BustaNetMessages.h
 *
 * @brief Server notification of a client's cursor location in character select mode
 *
 * The server uses this message when all characters are in character selection mode.  A
 *	client may see the cursors of all other opponents when character selections are made.
 *	This message tells a client the location of another client's cursor.
 */
struct sc_character_select_cursor
{
	char type;				/**< The message type */
	int length;				/**< The length of the entire message in bytes */
	int playerID;			/**< The identification number of the cursor's player */
	char cursorLoc;			/**< The location of the player's cursor */
};

/**
 * @struct sc_new_connection BustaNetMessages.h
 *
 * @brief Server notification of a newly connected client
 *
 * This message is sent from the server to a notify a client that a new client has
 *	connected to the server and has joined the game.
 */
struct sc_new_connection
{
	char type;				/**< The message type */
	int length;				/**< The length of the entire message in bytes */
	int playerID;			/**< The Identification number of the new player */
	char* playerName;		/**< The name of the new player */
};

/**
 * @struct sc_opponent_states BustaNetMessages.h
 *
 * @brief Server message containing in-game data
 *
 * The server sends this message out to connected clients when the game is in progress.
 *	The message contains information about a player's opponents.
 */
struct sc_opponent_states
{
	char type;				/**< The message type */
	int length;				/**< The length of the entire message in bytes */
	// TBI
};

/**
 * @struct sc_player_disconnect BustaNetMessages.h
 *
 * @brief Server notification that a player has disconnected
 *
 * The server sends out this message when a client has disconnected.  The message contains
 *	the identification number of the disconnected client as well as the reason for
 *	disconnection.
 */
struct sc_player_disconnect
{
	char type;				/**< The message type */
	int length;				/**< The length of the entire message in bytes */
	int playerID;			/**< The identification number of the disconnect client */
	char* discMessage;		/**< The disconnection reason */
};

/**
 * @struct sc_player_lost BustaNetMessages.h
 *
 * @brief Server notification that a player has lost
 *
 * This message is sent out when a player loses in the current multiplayer game.  The
 *	message identifies the player that has lost.
 */
struct sc_player_lost
{
	char type;				/**< The message type */
	int length;				/**< The length of the entire message in bytes */
	int playerID;			/**< The identification number of the player who lost */
};

/**
 * @struct sc_sandwich_send BustaNetMessages.h
 *
 * @brief Server notification of a sent special sandwich
 *
 * The server sends this message out when it needs to notify a client that a special
 *	sandwich has been sent between players.  The message marks the player that sent
 *	the sandwich, the player to receive the sandwich, and the type of the sandwich
 *	sent.
 */
struct sc_sandwich_send
{
	char type;				/**< The message type */
	int length;				/**< The length of the entire message in bytes */
	int targetPlayer;		/**< The identification number of the player who receives the sandwich */
	int originatingPlayer;	/**< The identification number of the player who sent the sandwich */
	// TBI - sandwich
};

/**
 * @struct sc_server_settings BustaNetMessages.h
 *
 * @brief Server settings message
 *
 * This message contains information about the server and its settings.
 */
struct sc_server_settings
{
	char type;				/**< The message type */
	int length;				/**< The length of the entire message in bytes */
	// TBI - server characteristics
};

/**
 * @struct sc_start_game BustaNetMessages.h
 *
 * @brief Server notification to start the game
 *
 * The server sends this message to a client to notify it of the beginning of the
 *	multiplayer game.
 */
struct sc_start_game
{
	char type;				/**< The message type */
	int length;				/**< The length of the entire message in bytes */
};
