/*	Filename:		BustaNetServer.cpp
 *	Description:	Busta' Sandwich server module implementation
 *
 */

#include <stdlib.h>
#include "BustaNetServer.h"

BustaNetServer::BustaNetServer()
{
	/* Initialize variables */
	clients = NULL;			// No clients
	clientSocketSet = NULL;	// No socket set for clients
	maxClients = 0;			// No clients allowed
	numClients = 0;			// No clients currently connected
	serverName = NULL;		// No server name
	serverSocketSet = NULL;	// No socket set for server messages
	servsock = NULL;		// No socket for server messages

	debug = false;		// DEBUG - debug output modifier
}

BustaNetServer::~BustaNetServer()
{
	// Close tcp server socket
	SDLNet_TCP_Close(servsock);

	// Close tcp client sockets, and delete client list
	for (int i = 0; i < numClients; i++)
	{
		SDLNet_TCP_Close(clients[i]->socket);		// Close client socket
		delete(clients[i]);							// Delete client entry
	}

	// Shutdown SDL
	SDL_Quit();
}

char* BustaNetServer::init(Uint16 servport)
{
	client* oneClient;		// client record pointer

	// Create client list
	clients = (client**)(malloc(maxClients*(sizeof(*oneClient))));
	for (int i = 0; i < maxClients; i++)
	{
		// Create new client record
		oneClient = (struct client*)(malloc(sizeof(*oneClient)));

		// Set Client defaults
		oneClient->active = false;
		oneClient->id_number = 0;
		oneClient->name = NULL;
		oneClient->ready = false;
		oneClient->socket = NULL;

		// Add client record to client list
		clients[i] = oneClient;
	}

	// Initialize SDL
	if (SDL_Init(0) < 0)
		return SDL_GetError();		// Return error msg on SDL error

	// Initialize SDLnet
	if (SDLNet_Init() == -1) 
		return SDLNet_GetError();	// Return error msg on SDLNet error

	// Initialize SDL timer subsystem
	if (SDL_Init(SDL_INIT_TIMER) < 0)
		return SDL_GetError();		// Return error msg on SDL timer error

	// Allocate the socket set
	clientSocketSet = SDLNet_AllocSocketSet(1);		// client socket set
	serverSocketSet = SDLNet_AllocSocketSet(1);		// server socket set

	// If we cannot create the server or client socket sets, return the error msg
	if ((clientSocketSet == NULL) || (serverSocketSet == NULL)) 
		return SDLNet_GetError();
		
	// Create the server socket
	SDLNet_ResolveHost(&serverIP, NULL, servport);			// resolve the server's ip with the specified port

	// Open the server socket
	servsock = SDLNet_TCP_Open(&serverIP);

	// If we cannot create the server socket, return the error msg
	if (servsock == NULL) 
		return SDLNet_GetError();

	// Add the server socket to the server socket set
	SDLNet_TCP_AddSocket(serverSocketSet, servsock);

	// return null on no error
	return NULL;
}

void BustaNetServer::addClient(IPaddress IP, TCPsocket socket, int id, char* clientName)
{
	int clientRecord;		// Open client record index in client list

	// Locate an empty client record
	for (clientRecord = 0; clientRecord < maxClients; clientRecord++)
	{
		if (!(clients[clientRecord]->active))
			break;								// break out once an open slot is found
	}

	// Create a record of the client
	clients[clientRecord]->active = true;
	clients[clientRecord]->address = IP;
	clients[clientRecord]->char_selected = false;
	clients[clientRecord]->id_number = id;
	clients[clientRecord]->name = clientName;
	clients[clientRecord]->ready = false;
	clients[clientRecord]->socket = socket;

	// Add the client socket to the socket set
	SDLNet_TCP_AddSocket(clientSocketSet, clients[clientRecord]->socket);

	// DEBUG - output connection info for client
	if (debug)
	{
		printf("Client Connected\n");
		printf("\tAddress: %x\n", clients[clientRecord]->address.host);
		printf("\tID Number: %i\n", clients[clientRecord]->id_number);
		printf("\tName: %s\n", clients[clientRecord]->name);
		printf("\n");
	}
}

void BustaNetServer::setDebug(bool OnOff)
{
	debug = OnOff;
}

void BustaNetServer::setMaxClients(int maxC)
{
	maxClients = maxC;
}

int BustaNetServer::setStateAccept()
{
	int clientsRdy = 0;		// number of ready clients
	TCPsocket newsock;		// Socket to assign to new clients

	// Begin Accepting Connections
	for (;;)
	{
		/* If there are at least two clients who are ready, exit accept state */
		if (numClients >= 2)
		{
			for (int i = 0; i < maxClients; i++)		// Count number of ready clients
			{
				if (clients[i]->ready)
					clientsRdy++;
			}

			if (clientsRdy == numClients)				// If all clients ready, exit accept state
				return 1;
		}

		// Check for input on server socket
		SDLNet_CheckSockets(serverSocketSet, 0);

		// If input on socket, we have a connection
		if (SDLNet_SocketReady(servsock))
		{
			// Keep the socket used to connect to the server
			newsock = SDLNet_TCP_Accept(servsock);

			// DEBUG - Notify console of connection attempt
			if (debug)
				printf("Incoming Connection\n");
		}

		// Wait to poll socket again (80 ms)
		SDL_Delay(80);
	}

	// Error return
	return 0;
}

void BustaNetServer::setName(char* servname)
{
	serverName = servname;
}

int BustaNetServer::setStateCharSelect()
{
	return 0;
}

