/*	Filename:		BustaNetServer.h
 *	Description:	Busta' Sandwich server module
 *
 */

#ifndef BUSTA_SANDWICH_SERVER
#define BUSTA_SANDWICH_SERVER

#include "SDL.h"
#include "SDL_net.h"
#include <string.h>

/**
 * @struct client BustaNetServer.h
 *
 * @brief Representation of a client connected to the server
 *
 * The struct contains information regarding a client connected to the server.  This is used by the
 *	server to maintain an accurate listing of all connected clients.
 */
struct client
{
	bool active;		/**< Whether this client record is in use by a real client */
	bool ready;			/**< Whether a ready msg has been received from this client */
	bool char_selected;	/**< Whether the client has selected a character */
	IPaddress address;	/**< The ip address and port of the client */
	TCPsocket socket;	/**< The socket used to connect to this client */
	int id_number;		/**< The identification number of the client */
	char* name;			/**< The client's player's name */
};

/**
 * @class BustaNetServer
 * @brief Server network module for Busta'Sandwich
 *
 * Standalone server class for hosting and handling multiplayer games of Busta'Sandwich..
 */
class BustaNetServer
{
	public:
		/**
		 * @brief Constructor
		 *
		 */
		BustaNetServer();

		/**
		 * @brief Destructor
		 *
		 */
		~BustaNetServer();

		/**
		 * @brief Initialize the server
		 *
		 * Initializes SDL and SDLNet.  A socket to connect to the server is created for the server's
		 *	ip address on the specified port.
		 *
		 * @param servport The port number to host the game on
		 *
		 * @return Returns a error message if an error occurs during initialization.  On no error
		 *	null is returned.
		 */
		char* init(Uint16 servport);

		/**
		 * @brief Sends a connection response to a connecting client
		 *
		 * Responds to a client's connection attempt.  The client is sent a message detailing whether
		 *	or not the connection was allowed, a message describing why or why not the connection was
		 *	allowed, and the player's identification number if the connection was accepted.
		 *
		 * @param playerID Player's identification number on accepted connection
		 * @param response Response to the connection attempt.  True for allow, false for deny
		 * @param message Description of the server's response; why allowed or denied
		 */
		void sendAccept(int playerID, bool response, char* message);

		//sendCharacterSelectedAllO(int sel_client, CHARACTER);
		//sendCharacterSelectCursorAllO(int sel_client, CURSOR);

		/**
		 * @brief Notifies all connected clients that multiplayer character selection is taking place
		 *
		 * A message is sent to all clients connected to the server that all clients are to now select
		 *	a multiplayer character.
		 */
		void sendCharacterSelectModeAll();

		//sendConnectedAllO(CLIENT);

		/**
		 * @brief Send a disconnect notification to all connected clients
		 *
		 * Sends a notification message to all clients that are still connected when a client disconnects
		 *	from the server.  Includes details as to why client disconnected.
		 *
		 * @param disc_client Identification number of the disconnected client
		 * @param message Reason for client disconnection
		 */
		void sendDisconnectedAllO(int disc_client, char* message);

		//sendEndAll(GAMESTATS);

		/**
		 * @brief Sends a notification to all client when one client has lost in the current game
		 *
		 * A notification that a client has lost in the current multiplayer game is sent to all other
		 *	connected clients.  The player who has lost is not notified.
		 *
		 * @param lost_player Identification number of the player who has lost
		 */
		void sendPlayerLostAllO(int lost_player);

		//sendSandwichAllO(int target_client, int from_client, SANDWICH);

		/**
		 * @brief Sends server information to all connected clients
		 *
		 * A message containing the server settings and information is sent to all clients.
		 */
		void sendServerSetupAll();

		/**
		 * @brief Notifies all connected clients to start the multiplayer game
		 *
		 * A message is sent to all connected clients that the multiplayer game is now starting.
		 */
		void sendGameStartAll();

		/**
		 * @brief Sends client game state information to all connected clients
		 *
		 * Sends game state information to all connected clients.  This includes the brief information
		 *	required by each player as well as the detailed game state requested by each client.
		 */
		void sendUpdateAll();

		/**
		 * @brief Sets the server in debug mode
		 *
		 * Turns on debugging output to the console for the server.
		 *
		 * @param OnOff Boolean to turn on or off debugging output
		 */
		void setDebug(bool OnOff);

		/**
		 * @brief Sets the maximum number of allowed connections
		 *
		 * Specifies the maximum number of simultaneous connections allowed by clients at any given time.
		 *
		 * @param maxC The maximum number of clients to allow
		 */
		void setMaxClients(int maxC);
		
		/**
		 * @brief Sets the name of the server
		 *
		 * Assigns the server an identification name.
		 *
		 * @param servname The name to give the server
		 */
		void setName(char* servname);

		/**
		 * @brief Sets the server in client connection accepting mode
		 *
		 * The server is now in a state to accept incoming TCP connections from clients.  The server may
		 *	leave this mode when all connected clients have sent a ready notification to the server and
		 *	there is at least two connected clients.
		 *
		 * @return Returns an integer greater than or equal to zero if the above condition is met.  A
		 *	negative integer is returned on error in this mode.
		 */
		int setStateAccept();

		/**
		 * @brief Sets the server in client character selection mode
		 *
		 * The server is now in a state that allows clients to select a multiplayer character.  The server
		 *	leaves this mode when all connected clients have sent a selected character to the server.  The server
		 *	may also leave this mode if client disconnects occur leaving less than two connected clients.
		 *
		 * @return Returns an integer greater than zero if all connected clients have selected a character.  Returns
		 *	zero if client disconnects leaves less than two players.  Returns a negative value on error.
		 */
		int setStateCharSelect();

		/**
		 * @brief Sets the server in game mode
		 *
		 * The server is now in multiplayer game mode.  All clients are currently playing the game.  The server
		 *	leaves this mode when all but one player has lost in the game or less than two clients remain connected
		 *	to the server while in this mode.
		 *
		 * @return Returns an integer greater than zero if a winner is determined.  Returns 0 if less than two clients
		 *	are connected and a negative value on error.
		 */
		int setStateGame();

	private:
		IPaddress serverIP;					// Server's IP address and port
		char* serverName;					// Server's name

		int maxClients;						// Maximum number of allow clients to the server
		int numClients;						// Number of clients currently connected to server
		client** clients;					// Array of all connected clients		

		SDLNet_SocketSet clientSocketSet;	// Socketset for client connections
		SDLNet_SocketSet serverSocketSet;	// Socketset for server socket
		TCPsocket servsock;					// Server socket for incoming connections

		void addClient(IPaddress IP, TCPsocket socket, int id, char* clientName);
		//sendCharacterSelected(int to_client, int sel_client, CHARACTER);
		//sendCharacterSelectCursor(int to_client, int sel_client, CURSOR);
		void sendCharacterSelectMode(int to_client);
		//sendConnected(int to_client, CLIENT);
		void sendDisconnected(int to_client, int disc_client);
		//sendEnd(int to_client, GAMESTATS);
		void sendPlayerLost(int to_client, int lost_player);
		//sendSandwich(int to_client, int target_client, int from_client, SANDWICH);
		void sendServerSetup(int to_client);
		void sendGameStart(int to_client);
		void sendUpdate(int to_client);

		bool debug;		// DEBUG NOTIFIER

};

#endif
